"""Database connection management"""

from typing import Optional
from sqlalchemy.orm import scoped_session
from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_sqlalchemy.session import Session
from .base import Base

DATABASE_FILENAME = "project.db"
db: Optional[SQLAlchemy] = None


def init_database(app: Flask, db_filename: str = DATABASE_FILENAME) -> SQLAlchemy:
    global db
    if db is None:
        db = SQLAlchemy(model_class=Base)
    db.init_app(app)
    with app.app_context():
        db.create_all()
    return db


def get_database() -> Optional[SQLAlchemy]:
    return db


def get_session() -> Optional[scoped_session[Session]]:
    return db.session if db else None



from src.opengeodeweb_microservice.database.connection import get_session
from src.opengeodeweb_microservice.database.data import Data


def test_data_crud_operations():
    data = Data.create(geode_object="test_object", input_file="test.txt")
    assert data.id is not None
    session = get_session()
    session.commit()
    retrieved = Data.get(data.id)
    assert retrieved is not None
    assert retrieved.geode_object == "test_object"
    non_existent = Data.get("fake_id")
    assert non_existent is None


def test_data_with_additional_files():
    files = ["file1.txt", "file2.txt"]
    data = Data.create(geode_object="test_files", additional_files=files)
    session = get_session()
    session.commit()
    retrieved = Data.get(data.id)
    assert retrieved.additional_files == files



from sqlalchemy import String, JSON
from sqlalchemy.orm import Mapped, mapped_column
from .connection import get_session
from .base import Base
import uuid


class Data(Base):
    __tablename__ = "datas"

    id: Mapped[str] = mapped_column(
        String, primary_key=True, default=lambda: str(uuid.uuid4()).replace("-", "")
    )
    native_file_name: Mapped[str] = mapped_column(String, nullable=False)
    viewable_file_name: Mapped[str] = mapped_column(String, nullable=False)
    geode_object: Mapped[str] = mapped_column(String, nullable=False)

    light_viewable: Mapped[str | None] = mapped_column(String, nullable=True)
    input_file: Mapped[str | None] = mapped_column(String, nullable=True)
    additional_files: Mapped[list[str] | None] = mapped_column(JSON, nullable=True)

    @staticmethod
    def create(
        geode_object: str,
        input_file: str | None = None,
        additional_files: list[str] | None = None,
    ) -> "Data":
        input_file = input_file if input_file is not None else ""
        additional_files = additional_files if additional_files is not None else []

        data_entry = Data(
            geode_object=geode_object,
            input_file=input_file,
            additional_files=additional_files,
            native_file_name="",
            viewable_file_name="",
            light_viewable=None,
        )

        session = get_session()
        if session:
            session.add(data_entry)
            session.flush()
            session.commit()
        return data_entry

    @staticmethod
    def get(data_id: str) -> "Data | None":
        session = get_session()
        if session:
            return session.get(Data, data_id)
        return None
